#!/bin/ksh

#
#
#
#
#
#

THISSCRIPT=`basename $0`
THISDIR=`dirname $0`
if [ "${THISDIR}" = "." ]; then
	THISDIR=`pwd`
fi

LogMsg()
{
	echo "$1"
}

LogError()
{
	echo "Error: $1"
}

#            1234567890123456789012345678901234567890123456789012345678901234567890123456789
UsageMsg()
{
	LogMsg	" "
	LogMsg	"Usage: ${THISSCRIPT} Command PatchName PatchHomeDirectory DataloadHomeDirectory"
	LogMsg	" "
	LogMsg	"Where:"
	LogMsg	" "
	LogMsg	"  Command is either 'prepare' or 'install'."
	LogMsg	"    'prepare' can be executed as regular pvuser. A successful 'prepare' phase"
	LogMsg	"    is mandatory for the 'install' phase to take place."
	LogMsg	"    'install' phase must be executed as root."
	LogMsg	" "
	LogMsg	"  PatchName is a a free text (no spaces) that will be used to create the name"
	LogMsg	"    of the backup directory."
	LogMsg	" "
	LogMsg	"  PatchHomeDirectory is the root of the directory, where the content of the"
	LogMsg	"    patch is available (usually a ReadOnly directory, either a CD, or a network"
	LogMsg	"    share, or a local copy in /tmp)"
	LogMsg	" "
	LogMsg	"  DataloadHomeDirectory is the current dataload root directory"
	LogMsg	"    (usually /opt/proviso/dataload; or /opt/dataload), that will be patched."	
	LogMsg	" "
	LogMsg	"Example:"
	LogMsg	" "
	LogMsg	"  ${THISSCRIPT} prepare 441FP001 /tmp/patch441FP001 /opt/dataload"
	LogMsg	" "
	LogMsg	"  ${THISSCRIPT} install 441FP001 /tmp/patch441FP001 /opt/dataload"
	return 0
}

CheckCommandLine()
{
##set -x

	if [ $# -ne 4 ]; then
		LogError "${THISSCRIPT}::CheckCommandLine( ) missing arguments."
		UsageMsg
		return 1
	fi

	COMMAND=$1
	PATCHNAME=`echo $2 | sed "s,/,,g"`
	PATCHHOME=`echo $3 | sed "s,/$,,g"`
	DLHOME=`echo $4 | sed "s,/$,,g"`

# Verify COMMAND is either prepare or install (undo will be supported later)
	if [ ! "${COMMAND}" = "prepare" -a ! "${COMMAND}" = "install" ]; then
		LogError "${THISSCRIPT}::CheckCommandLine( ) invalid 'Command' parameter."
		UsageMsg
		return 1
	fi

# Verify PATCHNAME is a valid name

# Verify PATCHHOME contains a valid patch structure, will all mandatory files
	if [ ! -f "${PATCHHOME}/version" ]; then
		LogError "${THISSCRIPT}::CheckCommandLine( ) '${PATCHHOME}/version' not found. Check 'PatchHomeDirectory' parameter."
		UsageMsg
		return 1
	fi

	if [ ! -f "${PATCHHOME}/contribs/${THISSCRIPT}" ]; then
		LogError "${THISSCRIPT}::CheckCommandLine( ) '${PATCHHOME}/contribs/${THISSCRIPT}' not found. Check 'PatchHomeDirectory' parameter."
		UsageMsg
		return 1
	fi

# Verify DLHOME contains a valid DL install
	FILE_PVM_OWNER=${DLHOME}/version
	if [ -f "${FILE_PVM_OWNER}" ]; then
		PVM_USER=`ls -o ${FILE_PVM_OWNER} | awk '{print $3}'`
		PVM_GROUP=`ls -g ${FILE_PVM_OWNER} | awk '{print $3}'`
   	else
		LogError "${THISSCRIPT}::CheckCommandLine( ) unable to determine Proviso owner from ${FILE_PVM_OWNER}."
		UsageMsg
		return 1
	fi

# Verify that the ${DLHOME}/version is of 'compatible' number
	LogMsg	"Current SNMP Dataload version is '`head -1 ${FILE_PVM_OWNER}`'"
	VALIDBASE=`cat ${FILE_PVM_OWNER} | grep "4.4.1" | wc -l`
	if [ ${VALIDBASE} -eq 0 ]; then
		LogError "This patch can only be applied on a 4.4.1(.x) version of Proviso Dataload"
		return 1
	fi
	


#   . $DATAHOME/dataLoad.env
#   VERSION_INFO=`$DATAHOME/bin/pvmd -v | xargs`
#   PVMD_VERSION=`echo $VERSION_INFO | sed -e "s/.*Comment: PV //g" | sed -e "s/ SNMP_DL.*//g"`
#   OS_VERSION=`echo $VERSION_INFO | awk '{print $2}'`
#   LogMsg "pvmd $PVMD_VERSION has support for Oracle $ORACLE_VERSION installed"

	return 0
}

systemcall()
{
   eval ${*}
   return=$?
   if [ ! "$return" = 0 ]; then
     LogError "Command '$*' Failed with shell error code $return, terminating."
     exit $return
   fi
}

addScriptHeader()
{
	COMMANDFILE=$1

	(
	cat <<-_END_
#!/bin/ksh

. ${DLHOME}/dataLoad.env

LogMsg()
{
	echo "\$1"
}

LogError()
{
	echo "Error: \$1"
}

systemcall()
{
   eval \${*}
   return=\$?
   if [ ! "\$return" = 0 ]; then
     LogError "Command '\$*' Failed with shell error code \$return, terminating."
     exit \$return
   fi
}

_END_
	) >> ${COMMANDFILE}
}

addToCommandFile()
{
	COMMANDFILE=$1
	COMMANDLINE=$2

	if [ ! -z "${COMMANDLINE}" ]; then
		echo systemcall "${COMMANDLINE}" >> ${COMMANDFILE}
	else
		echo >>  ${COMMANDFILE}
	fi
}


PatchPrepare()
{
#set -x 
	# 1)	Create patch/undo directory inside dataload structure
	# 2) 	establish list of files to backup
	# 3)	For each file, create the command journal, for install, and for undo

	BCKDIR=${DLHOME}/Patches/${PATCHNAME}
	if [ ! -d ${DLHOME}/Patches ]; then
		systemcall "mkdir ${DLHOME}/Patches"
	fi
	if [ ! -d ${BCKDIR} ]; then
		systemcall "mkdir ${BCKDIR}"
	fi

	INSTALLCOMMANDFILE=${BCKDIR}/install.${PATCHNAME}.sh
	if [ ! -f ${INSTALLCOMMANDFILE} ]; then
		systemcall "touch ${INSTALLCOMMANDFILE}"
	else
		systemcall "chown ${PVM_USER}:${PVM_GROUP} ${INSTALLCOMMANDFILE}"
		systemcall "chmod 750 ${INSTALLCOMMANDFILE}"
		systemcall "cp /dev/null ${INSTALLCOMMANDFILE}"
	fi
	systemcall "chown ${PVM_USER}:${PVM_GROUP} ${INSTALLCOMMANDFILE}"
	systemcall "chmod 750 ${INSTALLCOMMANDFILE}"
	addScriptHeader ${INSTALLCOMMANDFILE}

	UNDOCOMMANDFILE=${BCKDIR}/undo.${PATCHNAME}.sh
	if [ ! -f ${UNDOCOMMANDFILE} ]; then
		systemcall "touch ${UNDOCOMMANDFILE}"
	else
		systemcall "chown ${PVM_USER}:${PVM_GROUP} ${UNDOCOMMANDFILE}"
		systemcall "chmod 750 ${UNDOCOMMANDFILE}"
		systemcall "cp /dev/null ${UNDOCOMMANDFILE}"
	fi
	systemcall "chown ${PVM_USER}:${PVM_GROUP} ${UNDOCOMMANDFILE}"
	systemcall "chmod 750 ${UNDOCOMMANDFILE}"
	addScriptHeader ${UNDOCOMMANDFILE}

	#
	#	Loop thru all files within the patch tarball, and prepare the command scripts
	#	Some files, if distributed in the patch, carry extra operations, done at the end
	#
	NEEDALTERDLCFG=0
	NEEDALTERDATALOADENV=0
	NEEDALTERDATALOADCSHENV=0
	NEEDALTERINITD=0

	find ${PATCHHOME} | sed "s,${PATCHHOME},,g" | while read file; do
	echo "Preparing  ${file} ..."

		# if file fom the patch already exists in the DLHOME, file needs to be backed up
		#	1)	Make sure directory exists
		if [ -d ${DLHOME}${file} -a ! -d ${BCKDIR}${file} ]; then
			systemcall "mkdir ${BCKDIR}${file}"
		fi

		#	2)	if copy already exists .. removes copy
		if [ -f ${BCKDIR}${file} ]; then
			systemcall "rm -f ${BCKDIR}${file}"
		fi

		#	3)	copy file & make it read only
		if [ -f ${DLHOME}/${file} ]; then
			systemcall "cp -r ${DLHOME}${file} ${BCKDIR}${file}"
			systemcall "chmod 550 ${BCKDIR}${file}"
		fi

		# Detecting files, that will need special treatment later
		if [ "${file}" = "/contribs/dl.cfg.tmpl" ]; then
			NEEDALTERDLCFG=1
		fi

		if [ "${file}" = "/contribs/dataLoad.env.tmpl" ]; then
			NEEDALTERDATALOADENV=1
		fi

		if [ "${file}" = "/contribs/dataLoad.csh.env.tmpl" ]; then
			NEEDALTERDATALOADCSHENV=1
		fi

		if [ "${file}" = "/contribs/netpvmdrc.init" ]; then
			NEEDALTERINITD=1
		fi

		#
		# Here, add the file to the install journal
		#

		# make sure target directory exists
		if [ -d ${PATCHHOME}${file} ]; then
			if [ ! -d ${DLHOME}${file} ]; then
				addToCommandFile ${INSTALLCOMMANDFILE} "echo Creating directory ${file}"
				addToCommandFile ${INSTALLCOMMANDFILE} "mkdir  ${DLHOME}${file}"
				addToCommandFile ${INSTALLCOMMANDFILE} "chown ${PVM_USER}:${PVM_GROUP} ${DLHOME}${file}"
				addToCommandFile ${INSTALLCOMMANDFILE} ""
			fi
		fi

		if [ -f ${PATCHHOME}${file} ]; then

			if [ -f ${DLHOME}${file} ]; then
				addToCommandFile ${INSTALLCOMMANDFILE} "echo Updating ${file}"
				addToCommandFile ${INSTALLCOMMANDFILE} "chmod 750 ${DLHOME}${file}"
			else
				addToCommandFile ${INSTALLCOMMANDFILE} "echo Adding ${file}"
			fi
			addToCommandFile ${INSTALLCOMMANDFILE} "cp -f ${PATCHHOME}${file} ${DLHOME}${file}"

		#Handle qPing special case
			if [ "${file}" = "/bin/qPing" ]; then
				# special file
				addToCommandFile ${INSTALLCOMMANDFILE} "chown root:${PVM_GROUP} ${DLHOME}${file}"
				addToCommandFile ${INSTALLCOMMANDFILE} "chmod 6550 ${DLHOME}${file}"
			else
				# regular file
				addToCommandFile ${INSTALLCOMMANDFILE} "chown ${PVM_USER}:${PVM_GROUP} ${DLHOME}${file}"
				addToCommandFile ${INSTALLCOMMANDFILE} "chmod 550 ${DLHOME}${file}"
			fi
			addToCommandFile ${INSTALLCOMMANDFILE} ""
		fi

		# Here, add the file to the undo journal

	done

	# We systematically back up configuration and env files, even if unaltered by the patch
	if [ -d ${DLHOME}/conf -a ! -d ${BCKDIR}/conf ]; then
		systemcall "mkdir ${BCKDIR}/conf"
	fi

	if [ -f ${BCKDIR}/conf/dl.cfg ]; then
		systemcall "chmod 750 ${BCKDIR}/conf/dl.cfg"
	fi
	systemcall "cp ${DLHOME}/conf/dl.cfg ${BCKDIR}/conf/dl.cfg"
	systemcall "chmod 550 ${BCKDIR}/conf/dl.cfg"

	if [ -f ${BCKDIR}/dataLoad.env ]; then
		systemcall "chmod 750 ${BCKDIR}/dataLoad.env"
	fi
	systemcall "cp ${DLHOME}/dataLoad.env ${BCKDIR}/dataLoad.env"
	systemcall "chmod 550 ${BCKDIR}/dataLoad.env"

	if [ -f ${BCKDIR}/dataLoad.csh.env ]; then
		systemcall "chmod 750 ${BCKDIR}/dataLoad.csh.env"
	fi
	systemcall "cp ${DLHOME}/dataLoad.csh.env ${BCKDIR}/dataLoad.csh.env"
	systemcall "chmod 550 ${BCKDIR}/dataLoad.csh.env"

	# Here we add commands for smarter stuff, like upgrade/merge of the dl.cfg, merge of the dataLoad.env ...
	if [ "${NEEDALTERDLCFG}" = 1 ]; then
		echo "Preparing  /conf/dl.cfg ..."
		if [ -f ${BCKDIR}/conf/dl.cfg.merged ]; then
			systemcall "rm -f ${BCKDIR}/conf/dl.cfg.merged"
		fi
# we prefer to use the confMerge from the patch (should be available, if dl.cfg changes are required),
# otherwise, we use the one from the installed DL
		if [ -x ${PATCHHOME}/contribs/confMerge ]; then
			systemcall "${PATCHHOME}/contribs/confMerge -patch ${DLHOME}/conf/dl.cfg ${PATCHHOME}/contribs/dl.cfg.tmpl ${BCKDIR}/conf/dl.cfg.merged"
		else
			if [ -x ${DLHOME}/contribs/confMerge ]; then
				systemcall "${DLHOME}/contribs/confMerge -patch ${DLHOME}/conf/dl.cfg ${PATCHHOME}/contribs/dl.cfg.tmpl ${BCKDIR}/conf/dl.cfg.merged"
			else
				LogError "confMerge tool not found"
				return 1
			fi
		fi

		addToCommandFile ${INSTALLCOMMANDFILE} "echo Updating /conf/dl.cfg"
		addToCommandFile ${INSTALLCOMMANDFILE} "chmod 750 ${DLHOME}/conf/dl.cfg"
		addToCommandFile ${INSTALLCOMMANDFILE} "cp ${BCKDIR}/conf/dl.cfg.merged ${DLHOME}/conf/dl.cfg"
		addToCommandFile ${INSTALLCOMMANDFILE} "chown ${PVM_USER}:${PVM_GROUP} ${DLHOME}/conf/dl.cfg"
		addToCommandFile ${INSTALLCOMMANDFILE} "chmod 750 ${DLHOME}/conf/dl.cfg"
		addToCommandFile ${INSTALLCOMMANDFILE} ""
	fi

#	if [ "${NEEDALTERDATALOADENV}" = 1 ]; then
#
#	fi

#	if [ "${NEEDALTERDATALOADCSHENV}" = 1 ]; then
#
#	fi

#	if [ "${NEEDALTERINITD}" = 1 ]; then
#
#	fi

	# Here, since we now have SQL code to run as part of the migration, make sure than the runSql tool 
	# is available (preferably in the patch), otherwise in the installed DL, and that DB connectivity is available

	. ${DLHOME}/dataLoad.env

	if [ -x ${PATCHHOME}/contribs/runSql ]; then
		systemcall ${PATCHHOME}/contribs/runSql -probeDb -f ${DLHOME}/conf/dl.cfg
	else
		if [ -x ${DLHOME}/contribs/runSql ]; then
			systemcall ${DLHOME}/contribs/runSql -probeDb -f ${DLHOME}/conf/dl.cfg
		else
			LogError "runSql tool not found"
			return 1
		fi
	fi

	DB_VERSION=`${PATCHHOME}/contribs/runSql -sql "Select Pvm_Version.Component_Version_Info( 'DB', 'Database' ) From Dual" -sig 1660430468 -f ${DLHOME}/conf/dl.cfg |cut -d' ' -f1`
	if [ "${DB_VERSION}" = "4.4" ];then
		addToCommandFile ${INSTALLCOMMANDFILE} "echo DB version is 4.4.  Skipping installation of RA 502653."
	else
		addToCommandFile ${INSTALLCOMMANDFILE} "echo Patching database for RA 502653"
		addToCommandFile ${INSTALLCOMMANDFILE} "${DLHOME}/contribs/runSql -sql \"'DELETE FROM prop_desc WHERE idx_resource IN (SELECT DISTINCT(idx_resource) FROM prop_desc WHERE idx_metric=49000035 AND str_value IN (0,1)) AND idx_metric IN (49000034, 49000035, 49000036)'\" -f ${DLHOME}/conf/dl.cfg -sig 3354520548"	
		addToCommandFile ${INSTALLCOMMANDFILE} ""
	fi

	echo "return 0" >> ${INSTALLCOMMANDFILE}

	return 0
}

StopRunningCollector()
{
	# We prefer to use the pvmdmgr script provided with the patch (if available)
	# otherwise, we default to the one installed.
	#
	if [ -x ${PATCHHOME}/bin/pvmdmgr ]; then
		. ${DLHOME}/dataLoad.env; ${PATCHHOME}/bin/pvmdmgr stop
	else
		if [ -x ${DLHOME}/bin/pvmdmgr ]; then
			. ${DLHOME}/dataLoad.env; ${DLHOME}/bin/pvmdmgr stop
		else
			LogError "Unable to stop collector, missing or not executable pvmdmgr script"
		fi
	fi
	
	CheckWaitProcessDeath pvmd 125
	CheckWaitProcessDeath watchd 10
}

StartCollector()
{
	# We prefer to use the pvmdmgr script provided with the patch (if available)
	# otherwise, we default to the one installed.
	#
	if [ -x ${PATCHHOME}/bin/pvmdmgr ]; then
		. ${DLHOME}/dataLoad.env; ${PATCHHOME}/bin/pvmdmgr start
	else
		if [ -x ${DLHOME}/bin/pvmdmgr ]; then
			. ${DLHOME}/dataLoad.env; ${DLHOME}/bin/pvmdmgr start
		else
			LogError "Unable to start collector, missing or not executable pvmdmgr script"
		fi
	fi
}

CheckWaitProcessDeath()
{
#set -x
	
	PNAME=$1
	STOP_TIMEOUT=$2
	
	lstPid=`ps -ef | grep -w ${PNAME} | grep -v grep | nawk -F ' ' '{print $2}'`
	if [ ! -z "$lstPid" ]
	then
		nbs=0
		while [ $nbs -lt "$STOP_TIMEOUT" ]
		do
			lstPidToKill=
			for iPid in $lstPid
			do
				kill -0 $iPid >/dev/null 2>&1
				if [ $? -eq 0 ]
				then
					lstPidToKill="$lstPidToKill $iPid"
				fi
			done
			if [ "X$lstPidToKill" = "X" ]
			then
				break
			fi
			sleep 1
			nbs=`expr $nbs + 1`
		done
		
		if [ ! "X$lstPidToKill" = "X" ]
		then
			LogError "Program '${PNAME}' has not been terminated within the $STOP_TIMEOUT sec expected delay."
			LogError "A core file may be generated during upgrade."
		fi
	fi
}


PatchInstall()
{
	CURRENT_USER=`id | awk -F"(" '{print $2}' | awk -F")" '{print $1}'`
	if [ ! "$CURRENT_USER" = "root" ]; then
		LogError "Install MUST be run as root!"
   		return 1
	fi

	BCKDIR=${DLHOME}/Patches/${PATCHNAME}
	if [ ! -d ${BCKDIR} ]; then
		LogError "Missing Patch directory '${BCKDIR}'"
   		return 1
	fi

	INSTALLCOMMANDFILE=${BCKDIR}/install.${PATCHNAME}.sh
	if [ ! -f ${INSTALLCOMMANDFILE} ]; then
		LogError "Missing Install command script '${INSTALLCOMMANDFILE}'"
   		return 1
	fi

	LogMsg	"\n>> Stopping running collector ...\n"
	StopRunningCollector
	if [ ! $? -eq 0 ]; then
		LogError "Problem when stopping existing collector."
		return 1
	fi

	# Here, execute the command file
	LogMsg	"\n>> Executing the install patch command file ...\n"
	$INSTALLCOMMANDFILE
	if [ ! $? -eq 0 ]; then
		LogError "There has been a problem applying the patch (see previous error message)."
		return 1
	fi

	# here, check env:
	LogMsg	"\n>> Validating patched system ...\n"
	if [ -x ${DLHOME}/bin/checkSystem ]; then
		${DLHOME}/bin/checkSystem
	fi

	LogMsg	"\n>> Restarting collector ...\n"
	StartCollector
	if [ ! $? -eq 0 ]; then
		LogError "Problem when starting collector."
		return 1
	fi

	LogMsg	"\n>> Checking running version ...\n"
	sleep 10
	${DLHOME}/contribs/dialogTest2 version | grep "Version:"


	LogMsg	"\n>> Patch has been sucessfully applied.\n"
	return 0
}


#
#	Main
#

CheckCommandLine $*
if [ ! $? -eq 0 ]; then
	LogError "'CheckCommandLine $*' failed"
	return 1
fi

if [ "${COMMAND}" == "prepare" ]; then
	PatchPrepare
	if [ ! $? -eq 0 ]; then
		LogError "'PatchPrepare' failed"
		return 1
	fi
fi

if [ "${COMMAND}" == "install" ]; then
	PatchInstall
	if [ ! $? -eq 0 ]; then
		LogError "'PatchInstall' failed"
		return 1
	fi
fi

return 0
